// Edgardo Molina
// File: ImagePanel.cpp

#include <QtGui>

#include "ImagePanel.h"

// Constructor
ImagePanel::ImagePanel (QWidget* parent, Qt::WFlags f)
  : QWidget(parent, f) {
	// Set visual parameters
	QPalette pal;
	pal.setColor(QPalette::Window, QColor(Qt::black));
	pal.setColor(QPalette::WindowText, QColor(Qt::white));
	setPalette(pal);
	setBackgroundRole(QPalette::Window);
	setAutoFillBackground(true);
	setMouseTracking(true);
	_r = 0; _g = 0; _b = 0;
	setCursor(Qt::CrossCursor);
	
	// Set widget size
	resize(sizeHint());

	// reset will initialize some values
	reset();
}

// Destructor
ImagePanel::~ImagePanel() {
	// Currently nothing to destruct
}

// Init/Reset paramas
void
ImagePanel::reset() {
	// Initialize pixmap position
	_px = 0; _py = 0;
	
  	// initialize to empty pixmap
  	_pix = QPixmap();

	// Paint immediately
  	repaint();
}

// Shows an image on this widget
void
ImagePanel::show(QImage *img) {
	// Convert QImage to QPixmap for display purposes
  	_pix = QPixmap::fromImage(*img);
  	
	repaint();
}

// Called everytime a paint is needed
void
ImagePanel::paintEvent(QPaintEvent *) {
  QPainter painter(this);
  painter.setBackgroundMode(Qt::OpaqueMode);
  painter.setBackground(QBrush(Qt::black));

  // We draw the pixmap at position (_px,_py)
  painter.drawPixmap(_px,_py,_pix);

  // Extra credit
  // This is one way of displaying the color. Another way would be to add a
  // QLabel to MainWindow and display the color information there.
  painter.drawText(10, height()-10, "color = ("+QString::number(_r)+","+QString::number(_g)+","+QString::number(_b)+")");
}



// Below you will find the implementation of the mouse functions. Notice that
// e->button() and e->buttons() provide different information. Look at the
// documentation page: http://doc.trolltech.com/4.1/qmouseevent.html

// Qt::LeftButton and Qt::RightButton are not variables, they are enum's
// making them constants and should only be used as such.

// When you are drawing you need to tell the system to update the screen, we do this
// by calling repaint() or update(). repaint() will immediately paint to the screen.
// If you have complex drawing this may slow your application, it is often best to
// call update(), which paints to the screen at the next available time, and ignores
// very fast multiple calls. Look at the documentation for more info
// http://doc.trolltech.com/4.1/qwidget.html


void
ImagePanel::mousePressEvent(QMouseEvent* e) {
	if(e->button() == Qt::LeftButton) {
  		_pressed = true;
  		_x = e->x(); _y = e->y();
	}
	
	/**
	 * Here is a simple implementation to draw a square where the right mouse-button
	 * is pressed. There is a problem ... if the image is moved the squares will not
	 * be drawn where the user clicks. In addition a user may press outside of the
	 * image. The second implementation handles these situations.
	 *
	 * Notice that we can open a painter here because it is working on the image (&_pix)
	 * and not directly on the screen widget (this). But this will destroy image pixels,
	 * you could instead draw the squares to the screen leaving your image untouched.
	 * This choice is up to you and depends on your application needs.
	 */
	/* 
	if(e->button() == Qt::RightButton) {
		QPainter painter(&_pix);
		painter.fillRect(e->x()-2, e->y()-2, 5, 5, QBrush(Qt::green));
		
		update();
	}
	*/
	
	
	if(e->button() == Qt::RightButton) {
		// Adjust the mouse-point to the images coordinates
		int x = e->x() - _px;
		int y = e->y() - _py;
		
		QPainter painter(&_pix);
		// Check that the point is within the image space
		if(x < 0 || x > _pix.width()-5 || y < 0 || y > _pix.height()-5) return;
		painter.fillRect(x, y, 5, 5, QBrush(Qt::green));
		
		update();
	}
}

void
ImagePanel::mouseReleaseEvent(QMouseEvent* e) {
	if((e->button() == Qt::LeftButton) && _pressed) {
  		_pressed = false;
	}
}

void
ImagePanel::mouseMoveEvent(QMouseEvent* e) {
	if((e->buttons() == Qt::LeftButton) && _pressed) {
		int x = e->x();
		int y = e->y();

		// calculate the distance moved in x and y directions
		int dx = x - _x;
		int dy = y - _y;

		_x = x;
		_y = y;

		// update the pixmap location only,
		// then update(), we let the paint event handle the drawing
		_px += dx;
		_py += dy;
		update();
	}
	
	// Here is the implementation of the extra-credit.
	// setMouseTracking(true); must be set in the constructor for this to work.
	// This is not the only way to do this, but it is simple. We take a pixmap
	// of the widget, then we look and save the color at the current mouse location.
	// The drawing is done in the paintEvent() function.
	if(e->buttons() == Qt::NoButton) {
		QPixmap pic = QPixmap::grabWidget(this);
		QImage im = pic.toImage();
	  	QRgb color = im.pixel(e->x(), e->y());
	
		_r = qRed(color);
		_g = qGreen(color);
		_b = qBlue(color);
	
		update();
	}
}
